/*
 * AirTester v1.2
 * Author : (c) MadOrc madorc@gmx.com
 * ili9341 LIB (C) SHARANYADAS  http://community.atmel.com/projects/ili9341-library-drive-22-tft-displayderived-adafruit-tft-library-ili9340-type-controller
 * DHT22 LIB (c) Radu Motisan http://www.pocketmagic.net/temperature-and-humidity-sensor-dht-22/
 * BMP180 LIB (c)   http://www.avislab.com/blog/bmp085/
 */ 
#define F_CPU 12000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <math.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include "main.h"
#include "ili9341.h"
#include "ili9341gfx.h"
#include "dht22.h"
#include "atmega_bmp180_lib.h"

//  
#define CO2_WARM_UP 180 //   ,     - 3 
#define CO2_MEAS_PERIOD 10 //    co2   .       10.   -   .
#define FAN_ONCE_PER_SECOND 5 //    ,   
#define REDRAW_SENS_ONCE_PER_SECOND 20 //    .
#define VENTING_TIME_SECONDS  40 //  .        VENTING_LEVEL

#define VENTING_LEVEL 2000 //     PPM

//  
#define ALARM_SNOOZE_MIN  5 //        ALARM_SNOOZE_MIN 



#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)


#define color_ok	ILI9341_COLOR(0x49, 0xb8, 0x3f)
#define color_warn	ILI9341_COLOR(0xe1, 0xfa, 0x3c)
#define color_crit	ILI9341_COLOR(0xdd, 0x1e, 0x26)

#define color_co2_bg	ILI9341_COLOR(0xE3, 0xE3, 0xE3)
#define color_co2_txt	ILI9341_COLOR(0x00, 0x00, 0xF8)
#define color_co2_grid	ILI9341_COLOR(0x00, 0x00, 0x00)

#define color_temp_bg	ILI9341_COLOR(0xff, 0xed, 0x97)
#define color_temp_txt	ILI9341_COLOR(0x9b, 0x01, 0x42)

#define color_humi_bg	ILI9341_COLOR(0xcf, 0xe6, 0xff) 
#define color_humi_txt	ILI9341_COLOR(0x49, 0x4a, 0x1f)

#define color_press_bg	ILI9341_COLOR(0xba, 0xfc, 0xa8)
#define color_press_txt	ILI9341_COLOR(0x10, 0x10, 0x28)

#define HR 0
#define MI 1

uint16_t co2_min[86];
uint16_t div_graph_co2_minutes = 1;
uint32_t vl;
uint16_t color_tmp, color_tmp_tx, venting_in_progress = 0;

volatile uint8_t gre, grc;

signed char tempr_tmp;
uint8_t tmpr_min[86];
uint8_t humidity_min[86];

uint16_t pressure_min[86];

volatile uint8_t	minutes_counter=0,
					display_h_m=MI,
					do_beep=0;

uint8_t co2_average_measured=0;
uint8_t tmpr_average_measured=0;
uint8_t humidity_average_measured=0;
uint8_t pressure_average_measured=0;

uint16_t div_graph_co2_hours = 1;

uint16_t ppm_t=0;

uint16_t co2_average;
uint8_t tmpr_average;
uint8_t humidity_average;
uint16_t pressure_average;

uint16_t co2_hour[86];
uint8_t tmpr_hour[86];
uint8_t humidity_hour[86];
uint16_t pressure_hour[86];


volatile uint8_t alarm_snooze = 0, run_alarm = 0;


char msg[20];
unsigned char txt[10];

	int responseHigh;
	int responseLow;
	int ppm;
	unsigned char s;
	int i;
	float t = 0 , h = 0;
	
	
	int32_t temperature = 0;
	int32_t pressure = 0;
	int16_t BMP180_calibration_int16_t[8];
	int16_t BMP180_calibration_uint16_t[3];
	uint8_t error_code=0;


unsigned char cmd[9] = {0xFF,0x01,0x86,0x00,0x00,0x00,0x00,0x00,0x79};
unsigned char response[9];
volatile unsigned char symb, pos_in_readback;
unsigned char second=0, minute=26, hours=8, co2_alarm_level=0;

unsigned char beep_enabled=0;
unsigned int beep_counter;
volatile unsigned char	redraw_secondly=1, redraw_sensors=1, 
						redrawfull=1, redrawpartial=0, blink_clock=0, co2_started=CO2_WARM_UP,
						redraw_z19 = 1, redraw_dht22 = 1, redraw_bmp180 = 1,
						redraw_alarm = 1, blink_status=0, 
						do_measure_co2=1, do_measure_hum=1, do_measure_pres=1, do_shift_minutely=0, do_shift_hourly=0,
						sec_cnt=0, fan_sec_cnt=0, sens_sec_cnt=0,
						co2_warn_level, co2_warn_level_set = 6;
volatile uint16_t		dht22_valid = 0;
volatile uint16_t		ss, beep_time = 400;

volatile uint8_t		temp_index = 0, hum_index=0, temp_index_hour = 0, hum_index_hour=0, key=0;

volatile unsigned char co2_crc_ok=0;

extern uint16_t vsetx,vsety,vactualx,vactualy,isetx,isety,iactualx,iactualy;
static FILE mydata = FDEV_SETUP_STREAM(ili9341_putchar_printf, NULL, _FDEV_SETUP_WRITE);


//  Start

ISR(USART1_RX_vect){
	
	symb = UDR1;             //read UART register into value
	if (pos_in_readback<9) { response[pos_in_readback] = symb; pos_in_readback++; }
	
}


ISR(TIMER0_COMP_vect)
{ //
	
	if (co2_started) co2_started--;

	if ((venting_in_progress)&&(ppm < VENTING_LEVEL)) venting_in_progress--;
	if ((venting_in_progress == 2)||(co2_started)) Vent(0);
	
	if ((run_alarm)&&(!alarm_snooze)) do_beep = 1;
	Fan(0);
	redraw_secondly=1;
	second++;
	if(second == 60)        //  second = 60, second = 0
	{

		second = 0;
		minute++;            //  
		if (alarm_snooze) alarm_snooze--;
		//////////////////////////////
		do_shift_minutely = 1;
		minutes_counter++;
		if (minutes_counter >= 60)
		{
			minutes_counter = 0;
			do_shift_hourly = 1;
		}
		//////////////////////////////
		
	}

	if(minute == 60)        //  minute = 60, minute = 0
	{
		minute = 0;
		hours++;              //  
	}

	if(hours == 24)          //  hours = 24, hours = 0
	{
		hours = 0;
	}

if (sec_cnt++ > CO2_MEAS_PERIOD)
	{
do_measure_co2 = 1;
sec_cnt = 0;		
	}

if (fan_sec_cnt++ > FAN_ONCE_PER_SECOND)
{
	Fan(1);
	fan_sec_cnt = 0;
}

if (sens_sec_cnt++ > REDRAW_SENS_ONCE_PER_SECOND)
{
	redraw_sensors=1;
	sens_sec_cnt = 0;
}

do_measure_hum = 1;
do_measure_pres = 1;

} // 


ISR(TIMER2_COMP_vect)
{ //sound
	if (blink_status)
	{
		PORTB &= ~ (1<<PB5);
		PORTB |= (1<<PB4);
		blink_status=0;
	}
	else
	{
		PORTB |= (1<<PB5);
		PORTB &= ~ (1<<PB4);
		blink_status=1;
	}
	if (beep_counter++>beep_time)	{ TIMSK &=~ (1 << OCIE2); PORTB &= ~ (1<<PB4); PORTB &= ~ (1<<PB5); }
}

//  Stop



int main(void)
{
	pos_in_readback = 0;
	

// .   
	PORTB |= (0 << PB6);
	DDRB |=  (1 << DDB6);

// .   
PORTC |= (0 << PC7);
DDRC |=  (1 << DDC7);

//   
PORTB |= (0 << PB5)|(0 << PB4);
DDRB |=  (1 << DDB5)|(1 << DDB4);
	
//	PORTD |= (0 << PD3);
//	DDRD |=  (1 << DDD3);

//   
DDRA |= ((0 << PA0)|(0 << PA1)|(0 << PA2)|(0 << PA3));
PORTA |= ((1 << PA0)|(1 << PA1)|(1 << PA2)|(1 << PA3));
	
//	DDRD |= (0 << PD2);
//	PORTD |= (0 << PD2);
	
	stdout = & mydata;
	ili9341_init();//initial driver setup to drive ili9341
	ili9341_clear(BLACK);//fill screen with black color
	//_delay_ms(1000);
	ili9341_setRotation(0);//rotate screen
	_delay_ms(2);



	// Initialize BMP180
	BMP180_Calibration(BMP180_calibration_int16_t, BMP180_calibration_uint16_t,&error_code);


	// Use 8-bit character sizes:
	UCSR1C |= (1 << UCSZ10) | (1 << UCSZ11);
	// Load lower 8-bits of the baud rate value into the low byte
	// of the UBRR register:
	UBRR1L = BAUD_PRESCALE;
	// Load upper 8-bits of the baud rate value into the high byte
	// of the UBRR register:
	UBRR1H = (BAUD_PRESCALE >> 8);
	
	// Enable receiver and transmitter and receive complete interrupt
	UCSR1B = ((1<<TXEN1)|(1<<RXEN1) | (1<<RXCIE1));
	
		  
		  //
		  ASSR|=(1<<AS0); // 
		  TCCR0|=(1<<CS00)|(1<<CS01)|(1<<CS02)|(1<<WGM01)|(0<<WGM00); // 32768/1024 =32hz  2
		  OCR0=31; //  32-1

		  //  

		  TCCR3A = (1 << COM3A1)|(1 << WGM30);
		  TCCR3B = (1 << WGM32)|(1 << CS30);

		  //
		  TCCR2|=(1<<WGM21)|(0<<WGM20)|(0<<CS20)|(1<<CS21)|(0<<CS22); //
		  OCR2=205;
		  TCNT2=0;
		  
		  TIMSK |= (1 << OCIE0);

	sei();
	ili9341_clear(BLACK);//fill screen with black colour
	
	while(1)
	{

read_keys();

if (redraw_secondly)
{
//		ili9341_settextcolour(YELLOW,BLACK);

		if (blink_clock)
		{
			blink_clock = 0;
			sprintf (msg, "%02u %02u", hours, minute);
			//printf("%02u %02u", hours, minute);
		}
		
		else
		{
			blink_clock = 1;
			sprintf(msg, "%02u:%02u", hours, minute);
			//Beep();
		}



Special_Text(msg, 65,8, YELLOW, BLACK, (unsigned char *)term4_big_num_font);


redraw_secondly = 0;
}


if (do_shift_minutely)
	{
			
		if ((co2_crc_ok)&&(!(co2_started)))
			{


		div_graph_co2_minutes=1;
		for(uint8_t i=85;i>0;i--) // 
		{
			co2_min[i]=co2_min[i-1];
			if ((co2_min[i]>div_graph_co2_minutes)&&(i<85)) div_graph_co2_minutes=co2_min[i]; //  .
		}
			co2_min[0] = ppm;
			if (co2_average_measured) co2_average = (co2_average + ppm)/2; else co2_average = ppm;
			co2_average_measured=1;
			}

if (!dht22_valid) // 0=ok
	{
		for(uint8_t i=85;i>0;i--) //
		{
			tmpr_min[i]=tmpr_min[i-1];
			humidity_min[i]=humidity_min[i-1];
		}
		tempr_tmp = (signed char)t;
		if (tempr_tmp<0) tmpr_min[0]=0; 
		else if (tempr_tmp>60) tmpr_min[0] = 60; //   
		else tmpr_min[0] = (uint8_t)tempr_tmp;
		
		humidity_min[0] = (unsigned char)h;

		if (tmpr_average_measured) //  1   ,  0 ,   0
				tmpr_average = (tmpr_average + (uint8_t)tempr_tmp)/2; else tmpr_average = (uint8_t)tempr_tmp;
		if (humidity_average_measured)  //  1   ,  0 ,   0
				humidity_average = (humidity_average + (unsigned char)h)/2; else humidity_average = (unsigned char)h;

		if (temp_index < 85) temp_index++;
		if (hum_index < 85) hum_index++;
		
		tmpr_average_measured = 1;
		humidity_average_measured = 1;
		
		
	}

if (!error_code) // 0=ok
{
	for(uint8_t i=85;i>0;i--) //
	{
		pressure_min[i]=pressure_min[i-1];
	}
	pressure_min[0] = pressure/100;
	if (pressure_average_measured)
			pressure_average = (pressure_average + (pressure/100))/2; else pressure_average = (pressure/100);
	
	pressure_average_measured = 1;

}

do_shift_minutely=0;

// hourly  ,    
if (do_shift_hourly)
{
	div_graph_co2_hours=1;
	for(uint8_t i=85;i>0;i--) //
	{
		co2_hour[i]=co2_hour[i-1];
		if ((co2_hour[i]>div_graph_co2_hours)&&(i<85)) div_graph_co2_hours=co2_hour[i]; //  .
	}
	co2_hour[0] = co2_average;
	co2_average = 0;
	co2_average_measured=0;
	

	for(uint8_t i=85;i>0;i--) //
	{
		tmpr_hour[i]=tmpr_hour[i-1];
		humidity_hour[i]=humidity_hour[i-1];
	}
	tmpr_hour[0] = tmpr_average;
	humidity_hour[0] = humidity_average;
	tmpr_average_measured = 0;
	humidity_average_measured = 0;
	
	if (temp_index_hour < 85) temp_index_hour++;
	if (hum_index_hour < 85) hum_index_hour++;

	
	for(uint8_t i=85;i>0;i--) //
	{
		pressure_hour[i]=pressure_hour[i-1];
	}
	pressure_hour[0] = pressure_average;
	
	pressure_average_measured = 0;
	
	do_shift_hourly = 0;
}
// hourly

}


if (do_measure_co2) { do_measure_co2 = 0; Read_CO2(); redraw_z19 = 1; }

if (do_measure_hum) { do_measure_hum = 0; dht22_valid = dht22_read(&t,&h); redraw_dht22 = 1; }

if (do_measure_pres) { do_measure_pres = 0;  bmp180Convert(BMP180_calibration_int16_t, BMP180_calibration_uint16_t,&temperature, &pressure,&error_code); redraw_bmp180 = 1; }

	
if (redrawfull)
	{
		
		ili9341_fillrect(0, 49, 240, 64, color_co2_bg);

		ili9341_drawBitmap(5,54+8, co2_26x15, 26,15, color_co2_txt);
		

//		ili9341_drawBitmap(5,54+8, co2_26x15, 26,15, color_co2_txt);
//		ili9341_drawBitmap(5,96, sound_on_26x15, 26,15, color_crit);

		ili9341_settextcolour(color_co2_txt,color_co2_bg);

		ili9341_setcursor(34,96);
		ili9341_settextsize(1);
		printf("PPM");


		ili9341_settextcolour(color_co2_grid,color_co2_bg);
		ili9341_settextsize(1);
	
		ili9341_setcursor(57,102);
		printf("0");


		ili9341_setcursor(188,102);
		ili9341_settextsize(1);
	if (display_h_m == MI)
		printf(" ");
	else 
		printf("   ");


		ili9341_fillrect(0, 136, 240, 60, color_temp_bg);
		ili9341_drawBitmap(5,141+8, temp_26x15, 26,15, color_temp_txt);

		ili9341_settextcolour(color_co2_grid,color_temp_bg);
		ili9341_settextsize(1);

		ili9341_setcursor(188,102+84);
	if (display_h_m == MI)
	printf(" ");
	else
	printf("   ");

		ili9341_setcursor(57,102+84);
		printf("0");

		ili9341_setcursor(45,102+36);
		printf("+60");
		
		ili9341_fillrect(0, 198, 240, 60, color_humi_bg);
		ili9341_drawBitmap(5,203+8, humidity_26x15, 26,15, color_humi_txt);

		ili9341_settextcolour(color_co2_grid,color_humi_bg);
		ili9341_settextsize(1);

		ili9341_setcursor(188,102+84+62);
	if (display_h_m == MI)
	printf(" ");
	else
	printf("   ");

		ili9341_setcursor(57,102+84+62);
		printf("0");

		ili9341_setcursor(45,102+36+62);
		printf("100");




		
		ili9341_fillrect(0, 260, 240, 60, color_press_bg);
		ili9341_drawBitmap(3,265+8, pressure_26x15, 26,15, color_press_txt);
		
		ili9341_settextcolour(color_press_txt,color_press_bg);
		ili9341_setcursor(26,102+36+62+62+8+3);
		ili9341_settextsize(2);
		printf("hPa");
		
		ili9341_settextsize(1);
		ili9341_setcursor(57,102+84+62+62);
		printf("0");

		ili9341_setcursor(39,102+36+62+62);
		printf("1200");

		ili9341_setcursor(188,102+84+62+62);
	if (display_h_m == MI)
	printf(" ");
	else
	printf("   ");


		
		redrawfull = 0;
	}



if ((redraw_alarm)||(redrawfull))
{
	ili9341_fillrect(5,12, 26,15, BLACK);
	switch (co2_alarm_level)
	{
		case 0:
		default:
		color_tmp = ILI9341_COLOR(0x95,0x95,0x95);
		ppm_t = 0;
		break;
		case 1:
		color_tmp = ILI9341_COLOR(0x36,0x7a,0x2a);
		ppm_t = 700;
		break;
		case 2:
		color_tmp = ILI9341_COLOR(0x75,0x88,0x00);
		ppm_t = 1000;
		break;
		case 3:
		color_tmp = ILI9341_COLOR(0xea,0x82,0x26);
		ppm_t = 2500;
		break;
		case 4:
		color_tmp = ILI9341_COLOR(0xff,0x19,0x2a);
		ppm_t = 3500;
		break;
	}
	if (co2_alarm_level)
	ili9341_drawBitmap(5,12, sound_on_26x15, 26,15, color_tmp);
	else
	ili9341_drawBitmap(5,12, sound_off_26x15, 26,15, color_tmp);

	ili9341_settextcolour(color_tmp,BLACK);

	ili9341_setcursor(2,30);
	ili9341_settextsize(1);
	printf("%5u", ppm_t); // , co2_crc_ok
	

/*
	if (co2_alarm_level)
	ili9341_drawBitmap(2,54+6, sound_on_26x15, 26,15, color_tmp);
	else
	ili9341_drawBitmap(2,54+6, sound_off_26x15, 26,15, color_tmp);
*/

	redraw_alarm = 0;
}


if (redraw_z19)
	{
	ili9341_settextcolour(color_co2_txt,color_co2_bg);

	ili9341_setcursor(1,72+8);
	ili9341_settextsize(2);
	if	((!co2_crc_ok)&&((CO2_WARM_UP-co2_started)>(CO2_MEAS_PERIOD+5))) //
	printf("Error");
	else  if	(co2_started) // 0 = ready
	printf(" N/A ");
	else
	printf("%5u", ppm); // , co2_crc_ok




	if	((!co2_crc_ok)&&((CO2_WARM_UP-co2_started)>(CO2_MEAS_PERIOD+5))) // crc err
	{ co2_warn_level = 11; }
	else if	(co2_started) // 0 = ready
	{ co2_warn_level = 10; }
	else	if (ppm>3500) { co2_warn_level = 5; }
	else	if (ppm>2500) { co2_warn_level = 4; }
	else	if (ppm>1000) { co2_warn_level = 3; }
	else	if (ppm>700) { co2_warn_level = 2; }
	else	if (ppm>500) { co2_warn_level = 1; }
	else	{ co2_warn_level = 0; }

	if ((ppm > VENTING_LEVEL)&&(!venting_in_progress))
	
	{
		venting_in_progress = VENTING_TIME_SECONDS;
		Vent(1);
	}

	if (co2_warn_level != co2_warn_level_set)
	{
		co2_warn_level_set = co2_warn_level;

		run_alarm = 0; //   - 

		switch (co2_alarm_level)
		{
			case 0:
			default:
			break;
			
			case 1:
			if (ppm>700) { run_alarm = 1; }
			break;
			
			case 2:
			if (ppm>1000) { run_alarm = 1; }
			break;
			
			case 3:
			if (ppm>2500) { run_alarm = 1; }
			break;
			
			case 4:
			if (ppm>3500) { run_alarm = 1; }
			break;
			
		}


		switch (co2_warn_level)
		{
			case 0:
			color_tmp = ILI9341_COLOR(0x05,0x62,0x35); color_tmp_tx = WHITE;	strcpy(msg, "        ");
			break;
			case 1:
			color_tmp = ILI9341_COLOR(0x13,0x9b,0x49); color_tmp_tx = WHITE;	strcpy(msg, "   ");
			break;
			case 2:
			color_tmp = ILI9341_COLOR(0x50,0xb5,0x3f); color_tmp_tx = WHITE;	strcpy(msg, "     ");
			break;
			case 3:
			color_tmp = ILI9341_COLOR(0xe1,0xff,0x25); color_tmp_tx = BLACK;	strcpy(msg, ",  ");
			break;
			case 4:
			color_tmp = ILI9341_COLOR(0xea,0x62,0x26); color_tmp_tx = YELLOW;	strcpy(msg, "  !");
			break;
			case 5:
			color_tmp = ILI9341_COLOR(0xdf,0x19,0x2a); color_tmp_tx = YELLOW;	strcpy(msg, "! <8  ");
			break;
			case 10:
			color_tmp = ILI9341_COLOR(0x13,0x9b,0x49); color_tmp_tx = WHITE;	strcpy(msg, " ...  ");
			break;
			case 11:
			color_tmp = ILI9341_COLOR(0xed,0x62,0x26); color_tmp_tx = YELLOW;	strcpy(msg, " !     ");
			break;

		}
		ili9341_fillrect(0, 113, 240, 20, color_tmp);


		ili9341_setcursor(3,115);
		ili9341_settextsize(2);
		ili9341_settextcolour(color_tmp_tx,color_tmp);
		printf("%s", msg);
	}

	
	redraw_z19 = 0;
	}

if (redraw_dht22)
	{
	ili9341_settextcolour(color_temp_txt,color_temp_bg);

	ili9341_setcursor(1,80+87);
	ili9341_settextsize(2);
	if (!dht22_valid)
	{
		if ((t<0)||(t>99.99)) //  
		printf("%2.1f", t); //
		else
		printf("%2.2f", t); //
	}
	else
	printf("Error"); //


ili9341_settextcolour(color_humi_txt,color_humi_bg);

ili9341_setcursor(1,72+87+62+8);
ili9341_settextsize(2);

if (!dht22_valid)
printf("%2.2f", h); //
else
printf("Error"); //



	redraw_dht22 = 0;
	}


if (redraw_bmp180)
	{
	ili9341_settextcolour(color_press_txt,color_press_bg);

	ili9341_setcursor(1,72+87+62+62+8);
	ili9341_settextsize(2);

	if (!error_code)
	printf("%5ld", pressure/100);
	else
	printf("Error"); //

	redraw_bmp180 = 0;
	}

if (redraw_sensors)
{

gre = 0;
for (grc = 0; grc <170; grc++)
{

	ili9341_drawvline(64+grc,57,45, color_co2_bg);
		ili9341_drawvline(64+grc+1,57,45, color_co2_grid);
	
	if (display_h_m == MI)
	{//m/h
		if (co2_min[gre]) // != 0
		{
			vl = (uint32_t)co2_min[gre]*39;
			vl /= div_graph_co2_minutes;
			color_tmp = Crit_Color(co2_min[gre]);
			//ili9341_drawpixel(64+grc,64+(40-vl), color_tmp);
			if (vl < 40) ili9341_drawvline(64+grc,60+(39-vl),vl, color_tmp);
		}
	}//m/h
	else
	{//m/h
		if (co2_hour[gre]) // != 0
		{
			vl = (uint32_t)co2_hour[gre]*39;
			vl /= div_graph_co2_hours;
			color_tmp = Crit_Color(co2_hour[gre]);
			//ili9341_drawpixel(64+grc,64+(40-vl), color_tmp);
			if (vl < 40) ili9341_drawvline(64+grc,60+(39-vl),vl, color_tmp);
		}
	}//m/h
	
	if (!(grc%2))
	{
		gre++;
	ili9341_drawpixel(64+grc,72, color_co2_grid);

	ili9341_drawpixel(64+grc,86, color_co2_grid);
	}
	
	ili9341_drawpixel(64+grc,58, color_co2_grid);
	

	ili9341_drawpixel(64+grc,99, color_co2_grid);
	if (grc%20 == 0) // 20 = 10
		{ 
		for (ss = 58; ss < (58+43+1); ss +=2)
			ili9341_drawpixel(64+grc,ss, color_co2_grid);
		}
	
}
ili9341_drawvline(64+170,57,45, color_co2_bg);



ili9341_settextcolour(color_co2_grid,color_co2_bg);

ili9341_setcursor(33,51);
ili9341_settextsize(1);

if (display_h_m == MI)
	{//m/h
	printf("%5u", div_graph_co2_minutes);
	}//m/h
	else
	{//m/h
	printf("%5u", div_graph_co2_hours);
	}//m/h





////////////////////////////////////// TEMPERATURE

ili9341_settextcolour(color_temp_txt,color_temp_bg);

////////////////
gre = 0;
for (grc = 0; grc <170; grc++)
{

	ili9341_drawvline(64+grc,57+82,45, color_co2_bg);
		ili9341_drawvline(64+grc+1,57+82,45, color_co2_grid);

if (display_h_m == MI)
{//m/h

	if (gre < temp_index)
		{

			vl = (uint32_t)tmpr_min[gre]*39;

			vl /= 60;
			if (vl < 40) 
				{
				ili9341_drawpixel(64+grc,60+81+(39-vl), BLACK);
				ili9341_drawpixel(64+grc,60+81-1+(39-vl), RED);	
				}
		}
}//m/h
else
{//m/h

	if (gre < temp_index_hour)
	{

		vl = (uint32_t)tmpr_hour[gre]*39;

		vl /= 60;
		if (vl < 40)
		{
			ili9341_drawpixel(64+grc,60+81+(39-vl), BLACK);
			ili9341_drawpixel(64+grc,60+81-1+(39-vl), RED);
		}
	}
}//m/h

	if (!(grc%2))
	{
		gre++;
	ili9341_drawpixel(64+grc,72+82, color_co2_grid);

	ili9341_drawpixel(64+grc,86+82, color_co2_grid);
	}
	
	ili9341_drawpixel(64+grc,58+82, color_co2_grid);
	

	ili9341_drawpixel(64+grc,99+82, color_co2_grid);
	if (grc%20 == 0) // 20 = 10
		{ 
		for (ss = 58+82; ss < (58+43+87+1); ss +=2)
			ili9341_drawpixel(64+grc,ss, color_co2_grid);
		}
	
}
ili9341_drawvline(64+170,57+82,45, color_co2_bg);

ili9341_settextcolour(color_temp_txt,color_temp_bg);
////////////////////////////////////// /TEMPERATURE

//////////////////////////  HUMIDITY
ili9341_settextcolour(color_humi_txt,color_humi_bg);

////////////////
gre = 0;
for (grc = 0; grc <170; grc++)
{

	ili9341_drawvline(64+grc,57+82+62,45, color_co2_bg);
		ili9341_drawvline(64+grc+1,57+82+62,45, color_co2_grid);
	
if (display_h_m == MI)
{//m/h

	if (gre < hum_index)
		{

			vl = (uint32_t)humidity_min[gre]*39;
			vl /= 100;
			if (vl < 40) 
				{
				ili9341_drawpixel(64+grc,60+81+62+(39-vl), BLACK);
				ili9341_drawpixel(64+grc,60+81+62-1+(39-vl), BLUE);
				}
		}

}//m/h
else
{//m/h

	if (gre < hum_index_hour)
	{

		vl = (uint32_t)humidity_hour[gre]*39;
		vl /= 100;
		if (vl < 40)
		{
			ili9341_drawpixel(64+grc,60+81+62+(39-vl), BLACK);
			ili9341_drawpixel(64+grc,60+81+62-1+(39-vl), BLUE);
		}
	}

}//m/h

		
	if (!(grc%2))
	{
		gre++;
	ili9341_drawpixel(64+grc,72+82+62, color_co2_grid);

	ili9341_drawpixel(64+grc,86+82+62, color_co2_grid);
	}
	
	ili9341_drawpixel(64+grc,58+82+62, color_co2_grid);
	

	ili9341_drawpixel(64+grc,99+82+62, color_co2_grid);
	if (grc%20 == 0) // 20 = 10
		{ 
		for (ss = 58+82+62; ss < (58+43+87+62+1); ss +=2)
			ili9341_drawpixel(64+grc,ss, color_co2_grid);
		}
	
}
ili9341_drawvline(64+170,57+82+62,45, color_co2_bg);


//////////////////////////  /HUMIDITY


//////////////////////////  PRESSURE
ili9341_settextcolour(color_press_txt,color_press_bg);

////////////////
gre = 0;
for (grc = 0; grc <170; grc++)
{

	ili9341_drawvline(64+grc,57+82+62+62,45, color_co2_bg);
		ili9341_drawvline(64+grc+1,57+82+62+62,45, color_co2_grid);

if (display_h_m == MI)
{//m/h
if (pressure_min[gre])	// !=0= 
		{
			
			vl = pressure_min[gre];
			vl = (uint32_t)vl*39;
			vl /= 1200;
			if (vl < 40) 
				{
				ili9341_drawpixel(64+grc,60+81+62+62+(39-vl), BLACK);
				ili9341_drawpixel(64+grc,60+81+62+62-1+(39-vl), PURPLE);
				}
		}
}//m/h
else
{//m/h
	if (pressure_hour[gre])	// !=0= 
	{
		
		vl = pressure_hour[gre];
		vl = (uint32_t)vl*39;
		vl /= 1200;
		if (vl < 40)
		{
			ili9341_drawpixel(64+grc,60+81+62+62+(39-vl), BLACK);
			ili9341_drawpixel(64+grc,60+81+62+62-1+(39-vl), PURPLE);
		}
	}
}//m/h

	if (!(grc%2))
	{
		gre++;
	ili9341_drawpixel(64+grc,72+82+62+62, color_co2_grid);

	ili9341_drawpixel(64+grc,86+82+62+62, color_co2_grid);
	}
	
	ili9341_drawpixel(64+grc,58+82+62+62, color_co2_grid);
	

	ili9341_drawpixel(64+grc,99+82+62+62, color_co2_grid);
	if (grc%20 == 0) // 20 = 10
		{ 
		for (ss = 58+82+62+62; ss < (58+43+87+62+62+1); ss +=2)
			ili9341_drawpixel(64+grc,ss, color_co2_grid);
		}
	
}
ili9341_drawvline(64+170,57+82+62+62,45, color_co2_bg);


//////////////////////////  /PRESSURE



redraw_sensors = 0;
}

	if (do_beep)
		{
			Beep(1000);
			do_beep = 0;
		}

	}

}



uint16_t Crit_Color(uint16_t ppm_level)
{
	uint16_t c_tmp;
				if (ppm_level>3500) c_tmp = ILI9341_COLOR(0xdf,0x19,0x2a);
				else if (ppm_level>2500) c_tmp = ILI9341_COLOR(0xea,0x62,0x26);
				else if (ppm_level>1000) c_tmp = ILI9341_COLOR(0xbc,0xdb,0x00);
				else if (ppm_level>700) c_tmp = ILI9341_COLOR(0x50,0xb5,0x3f);
				else if (ppm_level>500) c_tmp = ILI9341_COLOR(0x13,0x9b,0x49);
				else c_tmp = ILI9341_COLOR(0x05,0x62,0x35);
	return c_tmp;
}


void Vent(uint8_t onoff)
{
	if (onoff)
	PORTC |= (1<<PC7);
	else
	PORTC &= ~ (1<<PC7);
}

void Fan(uint8_t onoff)
{
if (onoff)
	PORTB |= (1<<PB6);
	else
	PORTB &= ~ (1<<PB6);	
}

void Beep(uint16_t bttm)
{
	beep_time = bttm;
	beep_counter=0;
	TIMSK |= (1 << OCIE2);
}

void Read_CO2()
{
	pos_in_readback = 0;
	for (s=0;s<9;s++)
	{
		USART_Transmit(cmd[s]);
	}
	_delay_ms(1); //     
	responseHigh = (int) response[2];
	responseLow = (int) response[3];
	ppm = (256*responseHigh)+responseLow;
	//        PPM

  unsigned char crc = 0;
  for (i = 1; i < 8; i++) crc+=response[i];
  crc = 255 - crc;
  crc++;

  if ( !(response[0] == 0xFF && response[1] == 0x86 && response[8] == crc) )
	co2_crc_ok = 0;
  else
	co2_crc_ok = 1;

}

unsigned char USART_Receive( void )
{
	/* Wait for data to be received */
	while ( !(UCSR1A & (1<<RXC1)));
	/* Get and return received data from buffer */
	return UDR1;
}

void USART_Transmit( unsigned char data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR1A & (1<<UDRE1)) );
	/* Put data into buffer, sends the data */
	UDR1 = data;
}

void read_keys()
{

		
	
	if (key) return; //    ,   

	if((PINA&(1 << PA1))==0 )//  
	{
	key=1;
	Beep(400);
	_delay_ms(240);
	if ((run_alarm)&&(!alarm_snooze)) { alarm_snooze = ALARM_SNOOZE_MIN; key=0; return; }
	hours++;
	if (hours >=24) hours = 0;
	redraw_secondly = 1;

	key=0;
	}
else if((PINA&(1 << PA0))==0 )//  
	{
		key=1;
		Beep(400);
		_delay_ms(240);
		if ((run_alarm)&&(!alarm_snooze)) { alarm_snooze = ALARM_SNOOZE_MIN; key=0; return; }
		minute++;
		if (minute >=60) minute = 0;
		second = 0;
		redraw_secondly = 1;
		key=0;
	}
else if((PINA&(1 << PA2))==0 )//  
	{
		key=1;
		Beep(400);
		_delay_ms(550);
		if ((run_alarm)&&(!alarm_snooze)) { alarm_snooze = ALARM_SNOOZE_MIN; key=0; return; }
		if (co2_alarm_level < 4) co2_alarm_level++; else co2_alarm_level = 0;
		redraw_alarm = 1;
		key=0;
	}
else if((PINA&(1 << PA3))==0 )//  
	{
		key=1;
		Beep(400);
		_delay_ms(300);
		if ((run_alarm)&&(!alarm_snooze)) { alarm_snooze = ALARM_SNOOZE_MIN; key=0; return; }
		if (display_h_m) display_h_m = 0; else display_h_m = 1;
		redrawfull = 1;
		redraw_sensors = 1;
		redraw_alarm = 1;
		redraw_z19 = 1;
		redraw_dht22 = 1;
		redraw_bmp180 = 1;
		key=0;
	}
	

}
